/*
 * $Id: libbridge_init.c,v 1.3 2000/01/22 09:40:08 root Exp $
 *
 * Copyright (C) 2000 Lennert Buytenhek
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/fcntl.h>
#include <sys/ioctl.h>
#include "libbridge.h"

int br_socket_fd;
struct bridge *bridge_list;

void br_read_info(struct bridge *br)
{
	if_indextoname(br->ifindex, br->ifname);
	if (br_device_ioctl(br, BRCTL_GET_BRIDGE_INFO, (unsigned long)&br->info, 0, 0) < 0) {
		perror("br_read_info");
		exit(-1);
	}
}

void br_read_port_info(struct port *p)
{
	if (br_device_ioctl(p->parent, BRCTL_GET_PORT_INFO, (unsigned long)&p->info, p->index, 0) < 0) {
		perror("br_read_port_info");
		exit(-1);
	}
}

void br_make_port_list(struct bridge *br)
{
	int i;
	int ifindices[256];

	if (br_device_ioctl(br, BRCTL_GET_PORT_LIST, (unsigned long)ifindices, 0, 0) < 0) {
		perror("br_make_port_list");
		exit(-1);
	}

	for (i=255;i>=0;i--) {
		struct port *p;

		if (!ifindices[i])
			continue;

		p = malloc(sizeof(struct port));
		br->ports[i] = p;
		p->next = br->firstport;
		br->firstport = p;
		p->index = i;
		p->ifindex = ifindices[i];
		p->parent = br;

		br_read_port_info(p);
	}
}

void br_make_bridge_list()
{
	int i;
	int ifindices[32];
	int num;

	num = br_ioctl(BRCTL_GET_BRIDGES, (unsigned long)ifindices, 32);
	if (num < 0) {
		perror("br_make_bridge_list");
		return;
	}

	bridge_list = NULL;
	for (i=0;i<num;i++) {
		struct bridge *br;

		br = malloc(sizeof(struct bridge));
		memset(br, 0, sizeof(struct bridge));
		br->ifindex = ifindices[i];
		br->firstport = NULL;
		br->next = bridge_list;
		bridge_list = br;

		br_read_info(br);
		br_make_port_list(br);
	}
}

void br_init()
{
	if ((br_socket_fd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
		perror("opening AF_INET socket");
		exit(-1);
	}

	br_make_bridge_list();
}

void br_refresh()
{
	struct bridge *b;
	struct port *p;

	b = bridge_list;
	while (b != NULL) {
		struct bridge *bnext;

		p = b->firstport;
		while (p != NULL) {
			struct port *pnext;

			pnext = p->next;
			free(p);
			p = pnext;
		}

		bnext = b->next;
		free(b);
		b = bnext;
	}

	br_make_bridge_list();
}
